@@ -62,6 +62,11 @@ SUSPENDED_FAILS = 4
 SUSPENDED_REQUESTS = 5
 ACTIVE = 10
 
+def safe_int(x, default=-1):
+    """Return int(x) if possible, otherwise default (-1)."""
+    try: return int(x)
+    except (ValueError, TypeError): return default
+
 def increase(d, f, reset=False):
     """
     Increments a field `f` in the user dictionary `d`. Updates `ts_access` and `ts_login` if login route.
@@ -166,7 +171,7 @@ class Auth:
             ServiceException: If the database update fails.
         """
         res = await self.db.update(tbl, kc, d)
-        if res!=1: log_and_raise_service(self.LOG, label='AUTH', label2=prefix, msg='DB update error: {prefix}')
+        if res!=1: log_and_raise_service(self.LOG, label='AUTH', label2=prefix, msg=f'DB update error: {prefix}')
 
     async def enforce_find_user(self, d, label2=''):
         """
@@ -196,7 +201,7 @@ class Auth:
         """
         if not is_numeric(request.user.display_name):
             log_and_raise_service(self.LOG, label="AUTH", label2='enforce_admin_perm', msg="USER_NOT_REGISTERED", status_code=400)
-        user = await self.enforce_find_user({'id':int(request.user.display_name)}, label2=label2)
+        user = await self.enforce_find_user({'id':safe_int(request.user.display_name)}, label2=label2)
         role, org = user["role"], user["org"]
         roles = ["su"] if requires_su else ["su", "admin"]
         if user["ustatus"] != ACTIVE or role not in roles:
@@ -310,7 +315,7 @@ class Auth:
         if not is_numeric(request.user.display_name):
             log_and_raise_service(self.LOG,label="AUTH", label2=label2, msg="Invalid user.display_name", status_code=403)
         uid,payload = request.user.display_name, request.user.payload
-        user = await self.db.select_one("users", {"id":int(uid)})
+        user = await self.db.select_one("users", {"id":safe_int(uid)})
         if user is None:
             log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="USER_NOT_REGISTERED", status_code=401)
         if user["email"] != payload["email"]:
@@ -500,7 +505,7 @@ class Auth:
         label2 = "change_passwd"
         if not is_numeric(request.user.display_name):
             log_and_raise_service(self.LOG,label="AUTH", label2=label2, msg="USER_NOT_ACTIVE", status_code=403)
-        user = await self.enforce_find_user({"id":int(request.user.display_name)}, label2=label2)
+        user = await self.enforce_find_user({"id":safe_int(request.user.display_name)}, label2=label2)
         if user["ustatus"] == WAIT_SET_PASSWD:
             return self.ret(auth_status="WAIT_SET_PASSWD")
         if user["ustatus"] != ACTIVE:
@@ -612,7 +617,7 @@ class Auth:
     async def reset_user(self, request):
         """Reset user to SET_NEW_PASSWORD mode and reset fails."""
         label2 = "reset_user"
-        role, org = await self.enforce_admin_perm(request, label2=label2)
+        role,org = await self.enforce_admin_perm(request, label2=label2)
         data = await request.json()
         rec = rec_to_row(data)
         user = await self.db.select_one("users", {"id": rec["id"]})
@@ -635,7 +640,14 @@ class Auth:
             log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="USER_NOT_REGISTERED", status_code=400)
         if role == "admin" and user["org"] != org:
             log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="NOT_AUTHORIZED", status_code=403)
-        ustatus = int(rec["ustatus"]) if "ustatus" in rec and int(rec["ustatus"]) == SUSPENDED_LICENCE else SUSPENDED
+        ustatus = SUSPENDED
+        val = rec.get("ustatus")
+        if val is not None:
+            try:
+                v = safe_int(val)
+                ustatus = SUSPENDED_LICENCE if v==SUSPENDED_LICENCE else SUSPENDED
+            except (TypeError, ValueError):
+                pass
         user["ustatus"] = ustatus
         await self.db_update("users", "id", user, label2)
         return rest_ok()
@@ -689,6 +701,7 @@ class Auth:
 
         ##  make sure the jwt user exists
         if not is_numeric(request.user.display_name):
+<<<<<<< Updated upstream
             log_and_raise_service(self.LOG, label="AUTH", label2=label2,
                                   msg="USER_NOT_REGISTERED", status_code=400)
 
@@ -706,6 +719,12 @@ class Auth:
 
 
         # read target org from body
+=======
+            log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="USER_NOT_REGISTERED", status_code=400)
+        user = await self.db.select_one("users", {"id":safe_int(request.user.display_name)})
+        if user is None:
+            log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="USER_NOT_REGISTERED", status_code=400)
+>>>>>>> Stashed changes
         data = await request.json()
         target_org = oget(data, ["org"])
         if target_org is None:
@@ -721,7 +740,7 @@ class Auth:
 
         # if not su, validate if the target_org is on user_orgs
         if role != "su":
-            membership = await self.db.select_one("user_orgs", {"uid": uid, "org": target_org})
+            membership = await self.db.select_one("user_orgs", {"user_id": uid, "org_id": target_org})
             if membership is None:
                 log_and_raise_service(self.LOG, label="AUTH", label2=label2,
                                       msg="NOT_AUTHORIZED_FOR_ORG", status_code=403)
