@@ -9,8 +9,7 @@ The Auth routes are defined in the Auth module.
 The downstream services routes include:
     /api/contact
     /api/get_meta
-    /api/download_req
-    /api/upload_req
+    /api/get_signed_urls
     /api/download_workfile/{fname}
     /api/upload_workfile
     /api/publish_job
@@ -54,7 +53,7 @@ Every custom function implementing the route must observe the following:
        with HTTP status 400 and the error data as JSON
 
 Regarding files upload/downloand, consider the following:
-    1) The routes /api/download_req and /api/upload_req provice signed URLs for downnload/upload.
+    1) The route /api/get_signed_urls provide signed URLs for downnload/upload.
        The signed URLs can be obtained from the File Server or Google Cloud Storage.
        They are intended to offload the REST server of dealing with files, proxying to external services.
        The user namespace is limited to <bucket>.
@@ -66,24 +65,29 @@ The remaing methods deal with jobs lifecycle and there is a method to register c
 
 """
 from datetime import datetime, timezone
+from operator import itemgetter
+import os
 from pathlib import Path
 from starlette.responses import FileResponse
 from starlette.routing import Route
 from starlette.authentication import requires
-from dl2050utils.core import oget
+from dl2050utils.core import oget, LexIdx, get_uid, get_id
+from dl2050utils.gs import GS
 from dl2050utils.restutils import HTTPException, log_and_raise_exception, log_and_raise_rest, log_and_raise_service
 from dl2050utils.restutils import rest_ok,enforce_required_args, get_optional_args, form_files_upload
-from dl2050utils.restutils import fs_download_signed_url, fs_upload_signed_url, gcs_download_signed_url, gcs_upload_signed_url
+from dl2050utils.ulists import ulists_insert, ulists_update, ulists_delete, ulists_select
+from dl2050utils.ulists import ulist_items_insert, ulist_items_update, ulist_items_delete, ulist_items_select
 
 class App():
 
-    def __init__(self, cfg, LOG, NOTIFY, db, mq, auth, path, routes=[], appstartup=None, perm=None):
+    def __init__(self, cfg, LOG, NOTIFY, db, auth, path, routes=[], appstartup=None, perm=None):
         self.service = cfg['service']
         self.path = path
-        self.cfg,self.LOG,self.NOTIFY,self.db,self.mq,self.auth = cfg,LOG,NOTIFY,db,mq,auth
+        self.cfg,self.LOG,self.NOTIFY,self.db,self.auth = cfg,LOG,NOTIFY,db,auth
         self.routes,self.appstartup,self.perm = routes,appstartup,perm
-        self.d = {'LOG':LOG, 'path':path, 'db':db, 'mq':mq}
-        self.fs_url,self.fs_secret = oget(self.cfg,['fs','url']),oget(self.cfg,['fs','secret'])
+        self.d = {'cfg':cfg, 'LOG':LOG, 'path':path, 'db':db}
+        self.fs = GS(self.service) if oget(cfg,['gcloud','gs_key']) is not None else None
+        # self.fs_url,self.fs_secret = oget(self.cfg,['fs','url']),oget(self.cfg,['fs','secret'])
 
     # ####################################################################################################
     # Helper methods
@@ -103,11 +107,7 @@ class App():
     # ####################################################################################################
 
     async def startup(self):
-        model,meta = oget(self.cfg,['app','model']),None
-        # if model is not None: meta = await get_meta(self.path, self.db, model)
-        # self.d['model'] = model
-        # self.d['meta'] = meta
-        self.d['meta'] = None
+        self.d['meta'] = {}
         if self.appstartup is None: return False
         return await self.appstartup(self.d)
 
@@ -119,13 +119,18 @@ class App():
         BASE_ROUTES = [
             Route('/api/contact', endpoint=self.contact, methods=['POST']),
             Route('/api/get_meta', endpoint=self.get_meta, methods=['GET']),
-            Route('/api/download_req', endpoint=self.download_req, methods=['POST']),
-            Route('/api/upload_req', endpoint=self.upload_req, methods=['POST']),
+            Route('/api/get_signed_urls', endpoint=self.get_signed_urls, methods=['POST']),
             Route('/api/download_workfile/{fname:path}', endpoint=self.download_workfile, methods=['GET']),
             Route('/api/upload_workfile/{fname:path}', endpoint=self.upload_workfile, methods=['POST']),
-            Route('/api/publish_job', endpoint=self.publish_job, methods=['POST']),
-            Route('/api/get_jobs', endpoint=self.get_jobs, methods=['POST']),
-            Route('/api/get_job', endpoint=self.get_job, methods=['POST']),
+            # ULists
+            Route('/api/ulists_insert', endpoint=self.ulists_insert, methods=['POST']),
+            Route('/api/ulists_update', endpoint=self.ulists_update, methods=['POST']),
+            Route('/api/ulists_delete', endpoint=self.ulists_delete, methods=['POST']),
+            Route('/api/ulists_select', endpoint=self.ulists_select, methods=['POST']),
+            Route('/api/ulist_items_insert', endpoint=self.ulist_items_insert, methods=['POST']),
+            Route('/api/ulist_items_update', endpoint=self.ulist_items_update, methods=['POST']),
+            Route('/api/ulist_items_delete', endpoint=self.ulist_items_delete, methods=['POST']),
+            Route('/api/ulist_items_select', endpoint=self.ulist_items_select, methods=['POST']),
         ]
         APP_ROUTES = [Route(e, endpoint=self.app_route, methods=['POST']) for e in self.routes]
         return BASE_ROUTES + APP_ROUTES
@@ -134,6 +139,11 @@ class App():
     # REST methods for Apps
     # ####################################################################################################
 
+    @requires('authenticated')
+    async def get_meta(self, request):
+        """Returns the meta information."""
+        return rest_ok(self.d['meta'])
+    
     async def contact(self, request):
         """
         """
@@ -146,81 +156,36 @@ class App():
         html = '<h1>Name</h1><p>{d["email"]</p><h1>Email</h1><p>{d["email"]</p><h1>Message</h1><p>{d["msg"]</p>}'
         self.notify.send_mail_async(d['email'], subject='Contact from Website (cardiolife.global)', html=html)
         return rest_ok()
-
-    @requires('authenticated')
-    async def get_meta(self, request):
-        """Returns the meta information."""
-        return rest_ok(self.d['meta'])
     
     @requires('authenticated')
-    async def upload_req(self, request):
+    async def get_signed_urls(self, request):
         """
-            Returns an upload signed URL for <bucket>/<key> for the specified key.
-            The user namespace is limited to <bucket>.
-            The URL will be executed either by the File Server or the Cloud File Server (TO BE DEFINE HOW)
-            A 24h default timeout is set.
-            TODO: consider prefix <email> or <org>
-            TODO: Control size of user uploads
-            TODO: Resumable uploads with GCE
+        Provides signed urls for every path defined in the files parameter.
+        Keep the files tree structure on the File Server.
+        All paths will have the same uid prefix. A new uid is allways generated on this request.
+        Returns dict with two lists, the upload_urls and download_urls.
         """
-        if not self.fs_url or not self.fs_secret:
-            log_and_raise_service(self.LOG, label='REST', label2='download_req', msg='File server not defined')
-        u = await self.auth.check_auth(request)
-        uid = u['uid']
-        email = await self.auth.get_email_from_uid(uid)
-        await self.db_increase('users', 'id', u['uid'], 'uploads', 'uploads')
+        if self.fs is None:
+            log_and_raise_service(self.LOG, label='REST', label2='download_req', msg='Signed urls not available')
         data = await request.json()
-        args = ['bucket','key','size']
-        args = enforce_required_args(self.LOG, data, args, label='REST', label2='upload_req')
-        self.perm(self.d, u, request.url.path, args)
-        # Key is composed only by <key> for now, consideer <email><key> or <org><key> later
-        # key = f"{email}/{args['key']}"
-        key = args['key']
-        timeout = 7*24*3600
-        if self.fs_url!='gcs':
-            baseurl = f'https://{self.fs_url}'
-            url = fs_upload_signed_url(self.fs_secret, baseurl, args['bucket'], key, args['size'], timeout=timeout)
-            down_load_url = fs_download_signed_url(self.fs_secret, baseurl, args['bucket'], key, timeout=timeout)
-        else:
-            url = gcs_upload_signed_url(args['bucket'], args['key'], timeout=timeout, size=args['size'])
-            down_load_url = gcs_download_signed_url(args['bucket'], key, timeout=timeout)
-        self.LOG(2, 0, label='APP', label2='upload_req', msg={'url':url})
-        return rest_ok({'url':url, 'down_load_url':down_load_url, 'info':{'Host':oget(request.headers,['Host'])}})
-    
-    @requires('authenticated')
-    async def download_req(self, request):
-        """
-            Returns a download signed URL for <bucket>/<key> for the specified key.
-            The user namespace is limited to <bucket>/<email> and <email> is prepended to <key> as dir.
-            The URL will be executed either by the File Server or the Cloud File Server (TO BE DEFINE HOW)
-            A 24h default timeout is set.
-            TODO: consider prefix <email> or <org>
-        """
-        u = await self.auth.check_auth(request)
-        uid = u['uid']
-        email = await self.auth.get_email_from_uid(uid)
-        await self.db_increase('users', 'id', u['uid'], 'downloads', 'downloads')
-        data = await request.json()
-        args = ['bucket','key']
-        args = enforce_required_args(self.LOG, data, args, label='REST', label2='download_req')
-        self.perm(self.d, u, request.url.path, args)
-        # Key is composed only by <key> for now, consideer <email><key> or <org><key> later
-        # key = f"{email}/{args['key']}"
-        key = args['key']
-        timeout = 7*24*3600
-        if self.fs_url!='gcs':
-            baseurl = f'https://{self.fs_url}'
-            url = fs_download_signed_url(self.fs_secret, baseurl, args['bucket'], key, timeout=timeout)
-        else:
-            url = gcs_download_signed_url(args['bucket'], key, timeout=timeout)
-        self.LOG(2, 0, label='APP', label2='download_req', msg={'url':url})
-        return rest_ok({'url':url})
+        # files = get_param(request, 'files', list, data=data, LOG=self.LOG)
+        args = enforce_required_args(self.LOG, data, ['files'], label='REST', label2='get_signed_urls')
+        files = args['files']
+        uid = get_uid()
+        upload_urls,download_urls = [],[]
+        bucket = f'{self.service}-apiserver'
+        for file in files:
+            upload_url,download_url = self.fs.urls(bucket, f'{uid}/{file}')
+            if upload_url is None or download_url is None:
+                log_and_raise_service(self.LOG, label='REST', label2='get_signed_urls', msg='Signed urls not available')
+            upload_urls.append(upload_url),download_urls.append(download_url)
+        return rest_ok({'upload_urls':upload_urls, 'download_urls':download_urls})
 
     @requires('authenticated')
     async def download_workfile(self, request):
         """
             Downloads a workfile file fname belonging to a registered user.
-            Workfiles are usually results produced by workers (throught MQ).
+            Workfiles are usually results produced by workers.
             Workfiles are stored in the folder '$DATAPATH/workfiles/<service>.
         """
         u = await self.auth.check_auth(request)
@@ -248,62 +213,171 @@ class App():
             log_and_raise_service(self.LOG, label='REST', label2='upload_workfile', msg=f'Upload form error')
         await form_files_upload(self.LOG, form.items(), p, max_size=max_size)
         return rest_ok()
+    
+    # ####################################################################################################
+    # jobs
+    # ####################################################################################################
 
+    # @requires('authenticated')
+    # async def publish_job(self, request):
+    #     """
+    #         Publishes a new job, calling mq.publish after checking permissions.
+    #         Jobs require the user email as an attribute.
+    #     """
+    #     u = await self.auth.check_auth(request)
+    #     uid = u['uid']
+    #     email = await self.auth.get_email_from_uid(uid)
+    #     data = await request.json()
+    #     args = enforce_required_args(self.LOG, data, ['q','payload'], label='REST', label2='publish_job')
+    #     q,payload = args['q'],args['payload']
+    #     self.perm(self.d, u, request.url.path, args)
+    #     jid = await self.mq.publish(q, email, payload)
+    #     if jid is None :
+    #         log_and_raise_service(self.LOG, label='REST', label2='publish_job', msg='MQ publish error')
+    #     self.LOG(2, 0, label='APP', label2='publish_job')
+    #     return rest_ok({'jid': jid})
+
+    # @requires('authenticated')
+    # async def get_jobs(self, request):
+    #     """Gets all jobs from a user, optionaly filtered by qname, pending or jobs not done."""
+    #     u = await self.auth.check_auth(request)
+    #     uid = u['uid']
+    #     email = await self.auth.get_email_from_uid(uid)
+    #     data = await request.json()
+    #     kwargs = get_optional_args(data, ['qname','pending','not_done'])
+    #     jobs = await self.mq.get_jobs(email, **kwargs)
+    #     return rest_ok(jobs)
+
+    # @requires('authenticated')
+    # async def get_job(self, request):
+    #     """Gets a job details, identified by jid."""
+    #     u = await self.auth.check_auth(request)
+    #     data = await request.json()
+    #     args = enforce_required_args(self.LOG, data, ['jid'], label='REST', label2='get_job')
+    #     job = await self.mq.get_job(args['jid'])
+    #     return rest_ok(job)
+
+    # ####################################################################################################
+    # ulists
+    # ####################################################################################################
+    
     @requires('authenticated')
-    async def publish_job(self, request):
+    async def ulists_insert(self, request):
         """
-            Publishes a new job, calling mq.publish after checking permissions.
-            Jobs require the user email as an attribute.
+        Inserts new ulists record.
+        If ulist_code is provided and an ulist with the same ulist_code already exists, do not insert and
+        returns the existing UList record.
         """
         u = await self.auth.check_auth(request)
-        uid = u['uid']
-        email = await self.auth.get_email_from_uid(uid)
+        email = await self.auth.get_email_from_uid(u['uid'])
         data = await request.json()
-        args = enforce_required_args(self.LOG, data, ['q','payload'], label='REST', label2='publish_job')
-        q,payload = args['q'],args['payload']
-        self.perm(self.d, u, request.url.path, args)
-        jid = await self.mq.publish(q, email, payload)
-        if jid is None :
-            log_and_raise_service(self.LOG, label='REST', label2='publish_job', msg='MQ publish error')
-        self.LOG(2, 0, label='APP', label2='publish_job')
-        return rest_ok({'jid': jid})
+        args = enforce_required_args(self.LOG, data, ['ulist_code'], label='REST', label2='ulists_update')
+        ulist_code = itemgetter('ulist_code')(args)
+        kwargs = get_optional_args(data, ['ulist_name', 'ulist_payload'])
+        row = await ulists_insert(self.db, email, ulist_code, **kwargs)
+        if row is None: 
+            log_and_raise_rest(self.LOG, label='REST', label2='ulists_insert', status_code=400)
+        return rest_ok(row)
 
     @requires('authenticated')
-    async def get_jobs(self, request):
-        """
-            Gets all jobs from a user, optionaly filtered by qname, pending or jobs not done.
-        """
+    async def ulists_update(self, request):
+        """Gets the whole ulist, with 1024 limit."""
         u = await self.auth.check_auth(request)
-        uid = u['uid']
-        email = await self.auth.get_email_from_uid(uid)
+        email = await self.auth.get_email_from_uid(u['uid'])
         data = await request.json()
-        kwargs = get_optional_args(data, ['qname','pending','not_done'])
-        jobs = await self.mq.get_jobs(email, **kwargs)
-        return rest_ok(jobs)
+        args = enforce_required_args(self.LOG, data, ['ulist_code'], label='REST', label2='ulists_update')
+        ulist_code = itemgetter('ulist_code')(args)
+        kwargs = get_optional_args(data, ['ulist_name', 'ulist_payload'])
+        if await ulists_update(self.db, email, ulist_code, **kwargs): 
+            log_and_raise_rest(self.LOG, label='REST', label2='ulists_update', status_code=400)
+        return rest_ok()
+    
+    @requires('authenticated')
+    async def ulists_delete(self, request):
+        """Gets the whole ulist, with 1024 limit."""
+        u = await self.auth.check_auth(request)
+        email = await self.auth.get_email_from_uid(u['uid'])
+        data = await request.json()
+        args = enforce_required_args(self.LOG, data, ['ulist_code'], label='REST', label2='ulists_delete')
+        ulist_code = itemgetter('ulist_code')(args)
+        if await ulists_delete(self.db, email, ulist_code):
+            log_and_raise_rest(self.LOG, label='REST', label2='ulists_delete', status_code=400)
+        return rest_ok()
+    
+    @requires('authenticated')
+    async def ulists_select(self, request):
+        """Gets the whole ulist, with 1024 limit."""
+        u = await self.auth.check_auth(request)
+        email = await self.auth.get_email_from_uid(u['uid'])
+        data = await request.json()
+        kwargs = get_optional_args(data, ['ulist_code'])
+        rows = await ulists_select(self.db, email, **kwargs)
+        return rest_ok(rows)
+    
+    @requires('authenticated')
+    async def ulist_items_insert(self, request):
+        """ """
+        u = await self.auth.check_auth(request)
+        email = await self.auth.get_email_from_uid(u['uid'])
+        data = await request.json()
+        args = enforce_required_args(self.LOG, data, ['ulist_code'], label='REST', label2='ulist_items_insert')
+        ulist_code = itemgetter('ulist_code')(args)
+        kwargs = get_optional_args(data, ['item_name', 'item_payload', 'item_id_1', 'item_id_2'])
+        if await ulist_items_insert(self.db, email, ulist_code, **kwargs): 
+            log_and_raise_rest(self.LOG, label='REST', label2='ulist_items_insert', status_code=400)
+        return rest_ok()
 
     @requires('authenticated')
-    async def get_job(self, request):
-        """
-            Gets a job details, identified by jid.
-        """
+    async def ulist_items_update(self, request):
+        """Updates an ulist item"""
         u = await self.auth.check_auth(request)
+        email = await self.auth.get_email_from_uid(u['uid'])
         data = await request.json()
-        args = enforce_required_args(self.LOG, data, ['jid'], label='REST', label2='get_job')
-        job = await self.mq.get_job(args['jid'])
-        return rest_ok(job)
+        args = enforce_required_args(self.LOG, data, ['item_id'], label='REST', label2='ulists_update')
+        item_id = itemgetter('item_id')(args)
+        kwargs = get_optional_args(data, ['item_name', 'item_payload', 'item_id_1', 'item_id_2'])
+        if await ulist_items_update(self.db, email, item_id, **kwargs): 
+            log_and_raise_rest(self.LOG, label='REST', label2='ulist_items_update', status_code=400)
+        return rest_ok()
+    
+    @requires('authenticated')
+    async def ulist_items_delete(self, request):
+        """Delete an ulist item."""
+        u = await self.auth.check_auth(request)
+        email = await self.auth.get_email_from_uid(u['uid'])
+        data = await request.json()
+        args = enforce_required_args(self.LOG, data, ['item_id'], label='REST', label2='ulist_items_delete')
+        item_id = itemgetter('item_id')(args)
+        if await ulist_items_delete(self.db, email, item_id):
+            log_and_raise_rest(self.LOG, label='REST', label2='ulists_delete', status_code=400)
+        return rest_ok()
+    
+    @requires('authenticated')
+    async def ulist_items_select(self, request):
+        """Selects all items from ulist, with a 1024 limit."""
+        u = await self.auth.check_auth(request)
+        email = await self.auth.get_email_from_uid(u['uid'])
+        data = await request.json()
+        args = enforce_required_args(self.LOG, data, ['ulist_code'], label='REST', label2='ulist_items_select')
+        ulist_code = itemgetter('ulist_code')(args)
+        rows = await ulist_items_select(self.db, email, ulist_code)
+        return rest_ok(rows)
+    
+    # ####################################################################################################
+    # app_route
+    # ####################################################################################################
 
     @requires('authenticated')
     async def app_route(self, request):
-        """
-            Implements all the custom app routes.
-        """
+        """Implements all the custom app routes."""
+        label2 = f'app_route {request.url.path}'
         if request.url.path not in self.routes:
-            log_and_raise_rest(self.LOG, label='REST', label2=f'app_route {request.url.path}', msg='url path not found', status_code=400)
+            log_and_raise_rest(self.LOG, label='REST', label2=label2, msg='url path not found', status_code=400)
         d = self.routes[request.url.path]
         cb,args,kwargs = d['f'],d['args'],d['kwargs']
         u = await self.auth.check_auth(request)
         data = await request.json()
-        args2 = enforce_required_args(self.LOG, data, args, label='REST', label2=f'app_route {request.url.path}')
+        args2 = enforce_required_args(self.LOG, data, args, label='REST', label2=label2)
         kwargs2 = get_optional_args(data, kwargs)
         self.perm(self.d, u, request.url.path, {**args2, **kwargs2})
         try:
@@ -311,5 +385,5 @@ class App():
         except HTTPException as exc:
             raise
         except Exception as exc:
-            log_and_raise_exception(self.LOG, label='REST', label2='app_route', msg=str(exc))
+            log_and_raise_exception(self.LOG, label='REST', label2=label2, msg=str(exc))
         return rest_ok(res)
