@@ -1,3 +1,4 @@
+import sys
 import datetime
 import random
 import urllib
@@ -7,11 +8,12 @@ import hashlib
 import orjson
 import jwt
 import asyncpg
+import requests
 import aiohttp
 import starlette
 from starlette.responses import JSONResponse
-from google.cloud import storage
-from dl2050utils.core import oget
+# from google.cloud import storage
+from dl2050utils.core import oget, is_numeric_str
 from dl2050utils.fs import read_json
 
 # ####################################################################################################
@@ -86,15 +88,23 @@ def log_and_raise_rest(LOG, label='', label2='', msg='', status_code=500):
     """log_and_raise for REST application functions, defaulting to status 500."""
     return log_and_raise(LOG, error_type='REST', label=label, label2=label2, msg=msg, status_code=status_code)
 
+# To be completed and tested
+async def get_request_json(request, LOG=None, label=None, label2=None):
+    try:
+        data = await request.json()
+    except json.JSONDecodeError as e:
+        if LOG: log_and_raise_service(LOG, label=label, label2=label2, msg=f"Input JSON with errors", status_code=401)
+        return None
+    return data
+
 # ####################################################################################################
-# Args
+# REST params
 # ####################################################################################################
 
 def enforce_required_args(LOG, payload, args, label='', label2='', as_list=False):
     """ Returns dict (or list is as_list is True) with args present in payload. Raises REST exception if any arg is missing """
     args2,miss = {},[]
-    if payload is None:
-        return args2
+    if payload is None: return args2
     for e in args:
         if e not in payload:
             miss.append(e)
@@ -132,10 +142,55 @@ def sync_get_meta(path, db, model):
     return None
 
 # ####################################################################################################
-# Testing REST service requests from from client:
-#   rest_request, do_login
+# Requests
 # ####################################################################################################
 
+def sync_request(url, method='GET', headers=None, payload=None, LOG=None):
+    """
+    Performs a GET or POST request.
+    Always returns a dict with attrs status, message and optionally result.
+    """
+    if headers is None: headers = {}
+    try:
+        if method.upper() == 'GET':
+            res = requests.get(url, headers=headers, params=payload)
+            # print(res.url)
+        elif method.upper() == 'POST':
+            headers['Content-Type'] = 'application/json'
+            res = requests.post(url, headers=headers, json=payload)
+        else:
+            return {'status':400, 'message':f'Unsupported method: {method}'}
+        # Raises HTTPError for bad responses (4xx or 5xx)
+        res.raise_for_status()
+        if 'application/json' in res.headers.get('Content-Type', ''): return res.json()
+        return res.text
+    except requests.exceptions.HTTPError as exc:
+        if res.content:
+            try:
+                return res.json()
+            except ValueError as exc:
+                if LOG: LOG(4, 0, label='sync_request', label2='HTTPError', msg=f'{res.text}')
+                return {'status':500, 'message':f'{res.text}'}
+        else:
+            if LOG: LOG(4, 0, label='sync_request', label2='HTTPError', msg=f'{exc}')
+            return {'status':500, 'message':'HTTPError'}
+    except requests.exceptions.ConnectionError as exc:
+        if LOG:  LOG(4, 0, label='sync_request', label2='ConnectionError', msg=f'Connection error: {exc}')
+        return {'status': 500, 'message':'ConnectionError', 'details': str(exc)}
+    except Exception as exc:
+        if LOG: LOG(4, 0, label='sync_request', label2='EXCEPTION', msg=f'{exc}')
+        return {'status':500, 'message': f'Server Error'}
+
+# To review
+async def post_request(url, payload, headers={}):
+    async with aiohttp.ClientSession(headers=headers) as session:
+        response = await session.post(url, json=payload)
+        print(await response.json())
+
+async def post_request_sync(url, payload, headers={}):
+    res = requests.post(url, json=payload, headers=headers)
+
+# To review
 async def rest_request(session, url, headers, host, port, payload=None, method='POST', json=True):
     print(f'{method} url: http://{host}:{port}{url}')
     if method not in ['POST', 'GET']:
@@ -163,7 +218,8 @@ async def rest_request(session, url, headers, host, port, payload=None, method='
             return res
         else:
             return None
-        
+
+# To review      
 async def do_login(session, host, port, url, email, passwd):
     headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}
     payload = {'email': email, 'passwd': passwd}
@@ -261,7 +317,7 @@ async def form_files_upload(LOG, items, p, max_size=int(1e9)):
 
 # ################################################################################
 # Signed URLs for File Server download/upload: 
-#   gcs_download_signed_url, gcs_upload_signed_url
+#   fs_download_signed_url, fs_upload_signed_url
 # ################################################################################
 
 def fs_download_signed_url(secret, base_url, bucket, key, timeout=7*24*3600):
@@ -296,46 +352,48 @@ def fs_upload_signed_url(secret, base_url, bucket, key, size, timeout=7*24*3600)
 # ################################################################################
 # Signed URLs for Google Cloud Storage download/upload: 
 #   gcs_download_signed_url, gcs_upload_signed_url
+# TODO: Deprecate and replace by GS Class from fs.py
 # ################################################################################
 
-def gcs_download_signed_url(bucket_name, blob_name, timeout=15*60):
-    """
-        Generates a v4 signed URL for downloading a blob.
-        Signed url expires according to timeout (in seconds).
-        Requires a service account key json file (obtained in IAM).
-        Requires the ENV variable GOOGLE_APPLICATION_CREDENTIALS with this file path:
-            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = path_to_file
-        https://cloud.google.com/storage/docs/samples/storage-generate-signed-url-v4#storage_generate_signed_url_v4-python
-    """
-    try:
-        sc = storage.Client()
-        bucket = sc.bucket(bucket_name)
-        blob = bucket.blob(blob_name)
-        url = blob.generate_signed_url(version="v4", expiration=datetime.timedelta(seconds=timeout), method="GET")
-        return url
-    except Exception as exc:
-        print(f'EXCEPTION: {str(exc)}')
-        return None
 
-def gcs_upload_signed_url(bucket_name, blob_name, timeout=15*60, size=None):
-    """
-        Generates a v4 signed URL for uploading a blob using HTTP PUT.
-        Signed url expires according to timeout (in seconds).
-        Requires a service account key json file (obtained in IAM).
-        Requires the ENV variable GOOGLE_APPLICATION_CREDENTIALS with this file path:
-            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = path_to_file
-        https://cloud.google.com/storage/docs/samples/storage-generate-upload-signed-url-v4
-    """
-    try:
-        sc = storage.Client()
-        bucket = sc.bucket(bucket_name)
-        blob = bucket.blob(blob_name)
-        expiration = datetime.timedelta(seconds=timeout)
-        url = blob.generate_signed_url(version="v4", expiration=expiration, method="PUT", content_type="application/octet-stream")
-    except Exception as exc:
-        print(f'EXCEPTION: {str(exc)}')
-        return None
-    return url
+# def gcs_download_signed_url(bucket_name, blob_name, timeout=15*60):
+#     """
+#         Generates a v4 signed URL on GCS for downloading a blob.
+#         Signed url expires according to timeout (in seconds).
+#         Requires a service account key json file (obtained in IAM).
+#         Requires the ENV variable GOOGLE_APPLICATION_CREDENTIALS with this file path:
+#             os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = path_to_file
+#         https://cloud.google.com/storage/docs/samples/storage-generate-signed-url-v4#storage_generate_signed_url_v4-python
+#     """
+#     try:
+#         sc = storage.Client()
+#         bucket = sc.bucket(bucket_name)
+#         blob = bucket.blob(blob_name)
+#         url = blob.generate_signed_url(version="v4", expiration=datetime.timedelta(seconds=timeout), method="GET")
+#         return url
+#     except Exception as exc:
+#         print(f'EXCEPTION: {str(exc)}')
+#         return None
+
+# def gcs_upload_signed_url(bucket_name, blob_name, timeout=15*60, size=None):
+#     """
+#         Generates a v4 signed URL on GCS for uploading a blob using HTTP PUT.
+#         Signed url expires according to timeout (in seconds).
+#         Requires a service account key json file (obtained in IAM).
+#         Requires the ENV variable GOOGLE_APPLICATION_CREDENTIALS with this file path:
+#             os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = path_to_file
+#         https://cloud.google.com/storage/docs/samples/storage-generate-upload-signed-url-v4
+#     """
+#     try:
+#         sc = storage.Client()
+#         bucket = sc.bucket(bucket_name)
+#         blob = bucket.blob(blob_name)
+#         expiration = datetime.timedelta(seconds=timeout)
+#         url = blob.generate_signed_url(version="v4", expiration=expiration, method="PUT", content_type="application/octet-stream")
+#     except Exception as exc:
+#         print(f'EXCEPTION: {str(exc)}')
+#         return None
+#     return url
 
 # ####################################################################################################
 # Etc - to review
